@@ -16,11 +16,17 @@ setMethod("initialize","CuffFeature",
 				annotation=data.frame(),
 				fpkm=data.frame(),
 				diff=data.frame(),
+				repFpkm=data.frame(),
+				count=data.frame(),
+				genome=character(),
 				... ){
 			.Object<-callNextMethod(.Object,
 					annotation=annotation,
 					fpkm=fpkm,
 					diff=diff,
+					repFpkm=repFpkm,
+					count=count,
+					genome=genome,
 					...)				
 		}
 )
@@ -61,18 +67,39 @@ setMethod("length","CuffFeature",
 }
 setMethod("fpkm",signature="CuffFeature",.fpkm)
 
-.fpkmMatrix<-function(object){
+.fpkmMatrix<-function(object,sampleIdList){
+	#Sample subsetting
+	if(!missing(sampleIdList)){
+		if (!all(sampleIdList %in% samples(object))){
+			stop("Sample does not exist!")
+		}else{
+			mySamples<-sampleIdList
+		}
+	}else{
+		mySamples<-samples(object)
+	}
 	res<-fpkm(object)
 	colnames(res)[1]<-"tracking_id"
 	res<-res[,c(1:3)]
 	res<-melt(res)
 	res<-dcast(res,tracking_id~sample_name)
 	res<-data.frame(res[,-1],row.names=res[,1])
+	if(!missing(sampleIdList)){
+		res<-res[,mySamples]
+	}
 	res
 }
 
 setMethod("fpkmMatrix",signature(object="CuffFeature"),.fpkmMatrix)
 
+.samples<-function(object){
+	res<-fpkm(object)$sample_name
+	res
+}
+
+setMethod("samples","CuffFeature",.samples)
+
+
 #setMethod("diff","CuffFeature",function(object){
 #		return(object@diff)
 #		})
@@ -83,10 +110,26 @@ setMethod("fpkmMatrix",signature(object="CuffFeature"),.fpkmMatrix)
 
 setMethod("diffData",signature(object="CuffFeature"),.diffData)
 
-setMethod("annotation","CuffFeature",function(object){
+setMethod(BiocGenerics::annotation,signature(object="CuffFeature"),function(object){
 		return(object@annotation)
 		})
 
+setMethod("getGenome","CuffFeature",function(object){
+			return(object@genome)
+		})
+
+.repFpkm<-function(object){
+	object@repFpkm
+}
+
+setMethod("repFpkm",signature(object="CuffFeature"),.repFpkm)
+
+.count<-function(object){
+	object@count
+}
+
+setMethod("count",signature(object="CuffFeature"),.count)
+
 #################
 #Setters		#
 #################
@@ -95,13 +138,17 @@ setMethod("annotation","CuffFeature",function(object){
 #################
 #Plotting		#
 #################
-.barplot<-function(object,logMode=FALSE,pseudocount=1.0,showErrorbars=TRUE,showStatus=TRUE,...){
+.barplot<-function(object,logMode=FALSE,pseudocount=1.0,showErrorbars=TRUE,showStatus=TRUE,replicates=FALSE,...){
 	quant_types<-c("OK","FAIL","LOWDATA","HIDATA","TOOSHORT")
 	quant_types<-factor(quant_types,levels=quant_types)
 	quant_colors<-c("black","red","blue","orange","green")
 	names(quant_colors)<-quant_types
 	
 	dat<-fpkm(object)
+	if(replicates){
+		repDat<-repFpkm(object)
+		colnames(repDat)[1]<-"tracking_id"
+	}
 	#TODO: Test dat to ensure that there are >0 rows to plot.  If not, trap error and move on...
 	
 	colnames(dat)[1]<-"tracking_id"
@@ -111,12 +158,20 @@ setMethod("annotation","CuffFeature",function(object){
 	    dat$fpkm <- dat$fpkm + pseudocount
 	    dat$conf_hi <- dat$conf_hi + pseudocount
 	    dat$conf_lo <- dat$conf_lo + pseudocount
+		
+		if(replicates){
+			repDat$fpkm<-repDat$fpkm + pseudocount
+		}
     }
 
     p<-ggplot(dat,aes(x=sample_name,y=fpkm,fill=sample_name))
     
-	p <- p + 
-	    geom_bar()
+	p <- p + geom_bar()
+	
+	if(replicates){
+		p <- p + geom_point(aes(x=sample_name,y=fpkm),size=3,shape=18,colour="black",data=repDat)
+	}
+	
 	if (showErrorbars)
 	{
 	    p <- p +
@@ -129,7 +184,7 @@ setMethod("annotation","CuffFeature",function(object){
     }
 	
     p <- p + facet_wrap('tracking_id') +
-          opts(title=object@annotation$gene_short_name,axis.text.x=theme_text(hjust=0,angle=-90))
+          labs(title=object@annotation$gene_short_name)+theme(axis.text.x=element_text(hjust=0,angle=-90))
 	
     if (logMode)
     {
@@ -146,7 +201,7 @@ setMethod("annotation","CuffFeature",function(object){
 		}
 	}
 	
-	p <- p + opts(legend.position="none")
+	p <- p + theme(legend.position="none")
 	
 	#Default cummeRbund colorscheme
 	p<-p + scale_fill_hue(l=50,h.start=200)
@@ -160,7 +215,7 @@ setMethod("annotation","CuffFeature",function(object){
 setMethod("expressionBarplot",signature(object="CuffFeature"),.barplot)
 
 
-.expressionPlot<-function(object,logMode=FALSE,pseudocount=1.0, drawSummary=FALSE, sumFun=mean_cl_boot, showErrorbars=TRUE,showStatus=TRUE,...){
+.expressionPlot<-function(object,logMode=FALSE,pseudocount=1.0, drawSummary=FALSE, sumFun=mean_cl_boot, showErrorbars=TRUE,showStatus=TRUE,replicates=FALSE,...){
 	#Coloring scheme for quant flags
 	quant_types<-c("OK","FAIL","LOWDATA","HIDATA","TOOSHORT")
 	quant_types<-factor(quant_types,levels=quant_types)
@@ -169,16 +224,31 @@ setMethod("expressionBarplot",signature(object="CuffFeature"),.barplot)
 	
 	dat<-fpkm(object)
 	colnames(dat)[1]<-"tracking_id"
+	
+	if(replicates){
+		repDat<-repFpkm(object)
+		repDat$replicate<-as.factor(repDat$replicate)
+		colnames(repDat)[1]<-"tracking_id"
+	}
+	
 	if(logMode)
 	{
 	    dat$fpkm <- dat$fpkm + pseudocount
 	    dat$conf_hi <- dat$conf_hi + pseudocount
 	    dat$conf_lo <- dat$conf_lo + pseudocount
+		
+		if(replicates){
+			repDat$fpkm<-repDat$fpkm + pseudocount
+		}
     }
 	p <- ggplot(dat)
 	#dat$fpkm<- log10(dat$fpkm+pseudocount)
-	p <- p + 
-	    geom_line(aes(x=sample_name,y=fpkm,color=tracking_id,group=tracking_id))
+	p <- p + geom_line(aes(x=sample_name,y=fpkm,group=tracking_id,color=tracking_id))
+	
+	if(replicates){
+		p <- p + geom_point(aes(x=sample_name,y=fpkm,color=tracking_id),size=2.5,shape=18,data=repDat)
+	}
+	
 	if (showErrorbars)
 	{
 	    p <- p +
@@ -201,7 +271,7 @@ setMethod("expressionBarplot",signature(object="CuffFeature"),.barplot)
 	
 	if (logMode)
     {
-        p <- p + ylab(paste("Log10 FPKM + ",pseudocount))
+        p <- p + ylab(paste("FPKM + ",pseudocount))
     } else {
         p <- p + ylab("FPKM")
     }
@@ -210,12 +280,12 @@ setMethod("expressionBarplot",signature(object="CuffFeature"),.barplot)
 	p<-p + scale_fill_hue(l=50,h.start=200) + scale_color_hue(l=50,h.start=200)
 	
 	#Add Title
-	p<-p + opts(title=object@annotation$gene_short_name,axis.text.x=theme_text(hjust=0,angle=-90))
+	p<-p + labs(title=object@annotation$gene_short_name) + theme(axis.text.x=element_text(hjust=0,angle=-90))
 	
 	#Recolor quant flags
 	#for some reason this doesn't work (ggplot2 problem)
 	#p<- p+ scale_colour_manual(name='quant_status',values=quant_colors)
-	
+	p<-p+facet_wrap('tracking_id')
 	p
 }
 
