@@ -16,11 +16,15 @@ setMethod("initialize","CuffFeatureSet",
 				annotation=data.frame(),
 				fpkm=data.frame(),
 				diff=data.frame(),
+				repFpkm=data.frame(),
+				count=data.frame(),
 				... ){
 			.Object<-callNextMethod(.Object,
 					annotation=annotation,
 					fpkm=fpkm,
 					diff=diff,
+					repFpkm=repFpkm,
+					count=count,
 					...)
 		}
 )
@@ -39,10 +43,12 @@ setValidity("CuffFeatureSet",function(object){
 #Class Methods	#
 #################
 setMethod("show","CuffFeatureSet",function(object){
-		cat(class(object),"instance\nSlots:
+		cat(class(object),"instance for ",length(object)," features\nSlots:
 			\tannotation
 			\tfpkm
-			\tdiff\n")
+			\trepFpkm
+			\tdiff
+			\tcount\n")
 		}
 )
 
@@ -51,6 +57,10 @@ setMethod("length","CuffFeatureSet",
 			dim(x@annotation)[1]
 		}
 )
+
+#Add dim method so you can find number of samples in CuffFeatureSet, not just number of genes
+
+
 #################
 #Subsetting		#
 #################
@@ -70,28 +80,58 @@ setMethod("length","CuffFeatureSet",
 
 setMethod("samples","CuffFeatureSet",.samples)
 
+.replicates<-function(object){
+	res<-repFpkm(object)$rep_name
+	res
+}
+
+setMethod("replicates","CuffFeatureSet",.replicates)
+
 .fpkm<-function(object,features=FALSE){
+	myFPKM<-object@fpkm
+	myFPKM$stdev<-(myFPKM$conf_hi-myFPKM$fpkm)/2
 	if (features){
-		return (merge(object@annotation,object@fpkm))
+		return (merge(object@annotation,myFPKM,by=1))
 	}else{
-		return(object@fpkm)
+		return(myFPKM)
 	}
 }
 setMethod("fpkm",signature(object="CuffFeatureSet"),.fpkm)
 
+.repFpkm<-function(object,features=FALSE){
+	myFPKM<-object@repFpkm
+	#myFPKM$stdev<-(myFPKM$conf_hi-myFPKM$fpkm)/2
+	if (features){
+		return (merge(object@annotation,myFPKM,by=1))
+	}else{
+		return(myFPKM)
+	}
+}
+setMethod("repFpkm",signature(object="CuffFeatureSet"),.repFpkm)
+
 .featureNames<-function(object){
 	data.frame(tracking_id=object@annotation[,1],gene_short_name=object@annotation$gene_short_name)
 }
 
 setMethod("featureNames",signature(object="CuffFeatureSet"),.featureNames)
 
-.features<-function(object){
+.annotation<-function(object){
 	object@annotation
 }
 
-setMethod("features",signature(object="CuffFeatureSet"),.features)
+setMethod(BiocGenerics::annotation,signature(object="CuffFeatureSet"),.annotation)
 
-.fpkmMatrix<-function(object,fullnames=FALSE){
+.fpkmMatrix<-function(object,fullnames=FALSE,sampleIdList){
+	#Sample subsetting
+	if(!missing(sampleIdList)){
+		if (!all(sampleIdList %in% samples(object))){
+			stop("Sample does not exist!")
+		}else{
+			mySamples<-sampleIdList
+		}
+	}else{
+		mySamples<-samples(object)
+	}
 	if(fullnames){
 		res<-fpkm(object,features=TRUE)
 		res$tracking_id<-paste(res$gene_short_name,res[,1],sep="|")
@@ -104,20 +144,122 @@ setMethod("features",signature(object="CuffFeatureSet"),.features)
 	res<-melt(res)
 	res<-dcast(res,tracking_id~sample_name)
 	res<-data.frame(res[,-1],row.names=res[,1])
+	if(!missing(sampleIdList)){
+		res<-res[,mySamples]
+	}
 	res
 }
 
 setMethod("fpkmMatrix",signature(object="CuffFeatureSet"),.fpkmMatrix)
 
+.repFpkmMatrix<-function(object,fullnames=FALSE,repIdList){
+	#Sample subsetting
+	if(!missing(repIdList)){
+		if (!all(repIdList %in% replicates(object))){
+			stop("Replicate does not exist!")
+		}else{
+			myReps<-repIdList
+		}
+	}else{
+		myReps<-replicates(object)
+	}
+	if(fullnames){
+		res<-repFpkm(object,features=TRUE)
+		res$tracking_id<-paste(res$gene_short_name,res[,1],sep="|")
+	}else{
+		res<-repFpkm(object)
+		colnames(res)[1]<-"tracking_id"	
+	}
+	selectedRows<-c('tracking_id','rep_name','fpkm')
+	res<-res[,selectedRows]
+	res<-melt(res)
+	res<-dcast(res,tracking_id~rep_name)
+	res<-data.frame(res[,-1],row.names=res[,1])
+	if(!missing(repIdList)){
+		res<-res[,myReps]
+	}
+	res
+}
+
+setMethod("repFpkmMatrix",signature(object="CuffFeatureSet"),.repFpkmMatrix)
+
+.countMatrix<-function(object,fullnames=FALSE,sampleIdList){
+	#Sample subsetting
+	if(!missing(sampleIdList)){
+		if (!all(sampleIdList %in% samples(object))){
+			stop("Sample does not exist!")
+		}else{
+			mySamples<-sampleIdList
+		}
+	}else{
+		mySamples<-samples(object)
+	}
+	if(fullnames){
+		res<-count(object,features=TRUE)
+		res$tracking_id<-paste(res$gene_short_name,res[,1],sep="|")
+	}else{
+		res<-count(object)
+		colnames(res)[1]<-"tracking_id"	
+	}
+	selectedRows<-c('tracking_id','sample_name','count')
+	res<-res[,selectedRows]
+	res<-melt(res)
+	res<-dcast(res,tracking_id~sample_name)
+	res<-data.frame(res[,-1],row.names=res[,1])
+	if(!missing(sampleIdList)){
+		res<-res[,mySamples]
+	}
+	res
+}
+
+setMethod("countMatrix",signature(object="CuffFeatureSet"),.countMatrix)
+
+.repCountMatrix<-function(object,fullnames=FALSE,repIdList){
+	#Sample subsetting
+	if(!missing(repIdList)){
+		if (!all(repIdList %in% replicates(object))){
+			stop("Replicate does not exist!")
+		}else{
+			myReps<-repIdList
+		}
+	}else{
+		myReps<-replicates(object)
+	}
+	if(fullnames){
+		res<-repFpkm(object,features=TRUE)
+		res$tracking_id<-paste(res$gene_short_name,res[,1],sep="|")
+	}else{
+		res<-repFpkm(object)
+		colnames(res)[1]<-"tracking_id"	
+	}
+	selectedRows<-c('tracking_id','rep_name','external_scaled_frags')
+	res<-res[,selectedRows]
+	res<-melt(res)
+	res<-dcast(res,tracking_id~rep_name)
+	res<-data.frame(res[,-1],row.names=res[,1])
+	if(!missing(repIdList)){
+		res<-res[,myReps]
+	}
+	res
+}
+
+setMethod("repCountMatrix",signature(object="CuffFeatureSet"),.repCountMatrix)
+
 .diffData<-function(object){
 	return(object@diff)
 }
 
 setMethod("diffData",signature(object="CuffFeatureSet"),.diffData)
 
-#setMethod("diff","CuffFeatureSet",function(object){
-#			return(object@diff)
-#		})
+.count<-function(object,features=FALSE){
+	if (features){
+		return (merge(object@annotation,object@count,by=1))
+	}else{
+		return(object@count)
+	}
+}
+
+setMethod("count",signature(object="CuffFeatureSet"),.count)
 
 setMethod("annotation","CuffFeatureSet",function(object){
 			return(object@annotation)
@@ -134,7 +276,7 @@ setMethod("annotation","CuffFeatureSet",function(object){
 	}
 	colnames(dat)[1] <- "tracking_id"
 	p<-ggplot(dat)
-	p <- p + geom_tile(aes(x=tracking_id,y=sample_name,fill=fpkm)) + scale_fill_gradient(low="white",high="red") + opts(axis.text.x=theme_text(angle=-90, hjust=0))
+	p <- p + geom_tile(aes(x=tracking_id,y=sample_name,fill=fpkm)) + scale_fill_gradient(low="white",high="red") + theme(axis.text.x=element_text(angle=-90, hjust=0))
 	p
 }
 
@@ -145,15 +287,17 @@ setMethod("annotation","CuffFeatureSet",function(object){
 #There is no genericMethod yet, goal is to replace .heatmap with .ggheat for genericMethod 'csHeatmap'
 
 .ggheat<-function(object, rescaling='none', clustering='none', labCol=T, labRow=T, logMode=T, pseudocount=1.0, 
-		border=FALSE, heatscale=c(low='darkred',mid='orange',high='white'), heatMidpoint=NULL,fullnames=T,...) {
+		border=FALSE, heatscale=c(low='lightyellow',mid='orange',high='darkred'), heatMidpoint=NULL,fullnames=T,replicates=FALSE,method='none',...) {
 	## the function can be be viewed as a two step process
 	## 1. using the rehape package and other funcs the data is clustered, scaled, and reshaped
 	## using simple options or by a user supplied function
 	## 2. with the now resahped data the plot, the chosen labels and plot style are built
 
-	
-	m=fpkmMatrix(object,fullnames=fullnames)
-
+	if(replicates){
+		m=repFpkmMatrix(object,fullnames=fullnames)
+	}else{
+		m=fpkmMatrix(object,fullnames=fullnames)
+	}
 	#remove genes with no expression in any condition
 	m=m[!apply(m,1,sum)==0,]
 	
@@ -170,17 +314,17 @@ setMethod("annotation","CuffFeatureSet",function(object){
 	## if you want a different distance/cluster method-- or to cluster and then scale
 	## then you can supply a custom function 
 	
-	if(is.function(clustering)) 
-	{
-		m=clustering(m)
-	}else{
-		if(clustering=='row')
-			m=m[hclust(JSdist(makeprobs(t(m))))$order, ]
-		if(clustering=='column')  
-			m=m[,hclust(JSdist(makeprobs(m)))$order]
-		if(clustering=='both')
-			m=m[hclust(JSdist(makeprobs(t(m))))$order ,hclust(JSdist(makeprobs(m)))$order]
+	if(!is.function(method)){
+		method = function(mat){JSdist(makeprobs(t(mat)))}	
 	}
+
+	if(clustering=='row')
+		m=m[hclust(method(m))$order, ]
+	if(clustering=='column')  
+		m=m[,hclust(method(t(m)))$order]
+	if(clustering=='both')
+		m=m[hclust(method(m))$order ,hclust(method(t(m)))$order]
+
 	## this is just reshaping into a ggplot format matrix and making a ggplot layer
 	
 	if(is.function(rescaling))
@@ -217,7 +361,7 @@ setMethod("annotation","CuffFeatureSet",function(object){
 	## add the heat tiles with or without a white border for clarity
 	
 	if(border==TRUE)
-		g2=g+geom_rect(aes(xmin=colInd-1,xmax=colInd,ymin=rowInd-1,ymax=rowInd, fill=value),colour='white')
+		g2=g+geom_rect(aes(xmin=colInd-1,xmax=colInd,ymin=rowInd-1,ymax=rowInd, fill=value),colour='grey')
 	if(border==FALSE)
 		g2=g+geom_rect(aes(xmin=colInd-1,xmax=colInd,ymin=rowInd-1,ymax=rowInd, fill=value))
 	
@@ -243,15 +387,15 @@ setMethod("annotation","CuffFeatureSet",function(object){
 	}
 	
 	# Get rid of the ticks, they get way too dense with lots of rows
-    g2 <- g2 + opts(axis.ticks = theme_blank()) 
+    g2 <- g2 + theme(axis.ticks = element_blank()) 
 
 	## get rid of grey panel background and gridlines
 	
-	g2=g2+opts(panel.grid.minor=theme_line(colour=NA), panel.grid.major=theme_line(colour=NA),
-			panel.background=theme_rect(fill=NA, colour=NA))
+	g2=g2+theme(panel.grid.minor=element_line(colour=NA), panel.grid.major=element_line(colour=NA),
+			panel.background=element_rect(fill=NA, colour=NA))
 	
 	##adjust x-axis labels
-	g2=g2+opts(axis.text.x=theme_text(angle=-90, hjust=0))
+	g2=g2+theme(axis.text.x=element_text(angle=-90, hjust=0))
 
     #write(paste(c("Length of heatscale is :", length(heatscale))), stderr())
 	
@@ -268,7 +412,7 @@ setMethod("annotation","CuffFeatureSet",function(object){
 	} else if (length(heatscale) == 3) {
 	    if (is.null(heatMidpoint))
 	    {
-	        heatMidpoint = (max(m) - min(m)) / 2.0
+	        heatMidpoint = (max(m) + min(m)) / 2.0
 	        #write(heatMidpoint, stderr())
 	    }
 
@@ -285,8 +429,70 @@ setMethod("annotation","CuffFeatureSet",function(object){
 
 setMethod("csHeatmap",signature("CuffFeatureSet"),.ggheat)
 
+
+# Distance Heatmaps
+.distheat<-function(object, replicates=F, samples.not.genes=T, logMode=T, pseudocount=1.0, heatscale=c(low='lightyellow',mid='orange',high='darkred'), heatMidpoint=NULL, ...) {
+  # get expression from a sample or gene perspective
+	if(replicates){
+		obj.fpkm<-repFpkmMatrix(object,fullnames=T)
+	}else{
+		obj.fpkm<-fpkmMatrix(object,fullnames=T)
+	}
+
+	if(samples.not.genes) {
+    obj.fpkm.pos = obj.fpkm[rowSums(obj.fpkm)>0,]
+  } else {
+    obj.fpkm = t(obj.fpkm)
+    obj.fpkm.pos = obj.fpkm[,colSums(obj.fpkm)>0]
+  }
+  
+  if(logMode) {
+    obj.fpkm.pos = log10(obj.fpkm.pos+pseudocount)
+  }
+
+  # compute distances
+  obj.dists = JSdist(makeprobs(obj.fpkm.pos))
+  
+  # cluster to order
+  obj.hc = hclust(obj.dists)
+
+  # make data frame
+  dist.df = melt(as.matrix(obj.dists),varnames=c("X1","X2"))
+
+  # initialize
+  g = ggplot(dist.df, aes(x=X1, y=X2, fill=value))
+
+  # draw
+  labels = labels(obj.dists)
+  g = g + geom_tile() + scale_x_discrete("", limits=labels[obj.hc$order]) + scale_y_discrete("", limits=labels[obj.hc$order])
+
+  # roll labels
+  g = g + theme(axis.text.x=element_text(angle=-90, hjust=0), axis.text.y=element_text(angle=0, hjust=1))
+
+  # drop grey panel background and gridlines
+  g = g + theme(panel.grid.minor=element_line(colour=NA), panel.grid.major=element_line(colour=NA), panel.background=element_rect(fill=NA, colour=NA))
+
+  # adjust heat scale
+  if (length(heatscale) == 2) {
+    g = g + scale_fill_gradient(low=heatscale[1], high=heatscale[3], name="JS Distance")
+  }
+  else if (length(heatscale) == 3) {
+    if (is.null(heatMidpoint)) {
+      heatMidpoint = max(obj.dists) / 2.0
+    }
+    g = g + scale_fill_gradient2(low=heatscale[1], mid=heatscale[2], high=heatscale[3], midpoint=heatMidpoint, name="JS Distance")
+  }
+  
+  g <- g + geom_text(aes(label=format(value,digits=3)),size=6)
+  
+  # return
+  g
+}
+
+setMethod("csDistHeat", signature("CuffFeatureSet"), .distheat)
+
 #Scatterplot
-.scatter<-function(object,x,y,logMode=TRUE,pseudocount=1.0,labels, smooth=FALSE,colorByStatus=FALSE,...){
+.scatter<-function(object,x,y,logMode=TRUE,pseudocount=0.0,labels, smooth=FALSE,colorByStatus=FALSE,...){
 	dat<-fpkmMatrix(object)
 	samp<-samples(object)
 	
@@ -337,14 +543,14 @@ setMethod("csHeatmap",signature("CuffFeatureSet"),.ggheat)
     }
 	
 	#Add title & Return value
-	#p<- p + opts(title=object@tables$mainTable)
+	#p<- p + labs(title=object@tables$mainTable)
 	p
 }
 
 setMethod("csScatter",signature(object="CuffFeatureSet"), .scatter)
 
 #Volcano plot
-.volcano<-function(object,x,y,xlimits=c(-20,20),...){
+.volcano<-function(object,x,y,alpha=0.05,showSignificant=TRUE,xlimits=c(-20,20),...){
 	samp<-samples(object)
 	
 	#check to make sure x and y are in samples
@@ -352,6 +558,8 @@ setMethod("csScatter",signature(object="CuffFeatureSet"), .scatter)
 		stop("One or more values of 'x' or 'y' are not valid sample names!")
 	}
 	dat<-diffData(object=object)
+	dat$significant<-'no'
+	dat$significant[dat$q_value<=alpha]<-'yes'
 	
 	#subset dat for samples of interest
 	mySamples<-c(x,y)
@@ -362,22 +570,26 @@ setMethod("csScatter",signature(object="CuffFeatureSet"), .scatter)
 	s2<-unique(dat$sample_2)
 	
 	p<-ggplot(dat)
-	p<- p + geom_point(aes(x=log2_fold_change,y=-log10(p_value),color=significant),alpha=I(1/3))
+	if(showSignificant){
+		p<- p + geom_point(aes(x=log2_fold_change,y=-log10(p_value),color=significant),alpha=I(1/3),size=1.2)
+	}else{
+		p<- p + geom_point(aes(x=log2_fold_change,y=-log10(p_value)),alpha=I(1/3),size=1.2)
+	}
 	
-	p<- p + opts(title=paste(s2,"/",s1,sep=""))
+	p<- p + labs(title=paste(s2,"/",s1,sep=""))
 	
 	#Set axis limits
 	p<- p + scale_x_continuous(limits=xlimits)
 	
 	#Default cummeRbund colorscheme
-	p<-p + scale_color_hue(l=50,h.start=200)
+	p<-p + scale_colour_manual(values = c("black","red"))
 	
 	p
 }
 
 setMethod("csVolcano",signature(object="CuffFeatureSet"), .volcano)
 
-.barplot<-function(object,logMode=TRUE,pseudocount=1.0,showErrorbars=TRUE,showStatus=TRUE,...){
+.barplot<-function(object,logMode=TRUE,pseudocount=1.0,showErrorbars=TRUE,showStatus=TRUE,replicates=FALSE,...){
 	quant_types<-c("OK","FAIL","LOWDATA","HIDATA","TOOSHORT")
 	quant_types<-factor(quant_types,levels=quant_types)
 	quant_colors<-c("black","red","blue","orange","green")
@@ -388,9 +600,15 @@ setMethod("csVolcano",signature(object="CuffFeatureSet"), .volcano)
 	dat$warning[dat$quant_status!="OK"]<-"!"
 	#TODO: Test dat to ensure that there are >0 rows to plot.  If not, trap error and move on...
 	
+	#Handle replicates
+	if(replicates){
+		repDat<-repFpkm(object)
+		colnames(repDat)[1]<-"tracking_id"
+	}
+	
 	colnames(dat)[1]<-"tracking_id"
 	#tracking_ids<-dat$tracking_id
-	obj_features <- features(object)
+	obj_features <- annotation(object)
 	tracking_ids <- obj_features[,1]
 	
 	gene_labels<-obj_features$gene_short_name
@@ -408,13 +626,12 @@ setMethod("csVolcano",signature(object="CuffFeatureSet"), .volcano)
     }
 	
 	p<-ggplot(dat,aes(x=tracking_id,y=fpkm,fill=sample_name))
-	p <- p + 
-	    geom_bar(aes(group=1),position=dodge,stat='identity')
+	p <- p + geom_bar(position=dodge,stat='identity')
 
 	if (showErrorbars)
 	{
 	    p <- p +
-		    geom_errorbar(aes(ymin=conf_lo,ymax=conf_hi,group=1),position=dodge,width=0.5)
+		    geom_errorbar(aes(ymin=conf_lo,ymax=conf_hi),position=dodge,width=0.5)
 	}
 	
 	if (logMode)
@@ -424,20 +641,20 @@ setMethod("csVolcano",signature(object="CuffFeatureSet"), .volcano)
     }
 	if(showStatus){
 		if(logMode){
-			p <- p + geom_text(aes(x=tracking_id,y=1,label=warning,group=1),position=dodge,stat='identity',color='red',vjust=1.5,size=3)
+			p <- p + geom_text(aes(x=tracking_id,y=1,label=warning),position=dodge,stat='identity',color='red',vjust=1.5,size=3)
 		}else{
-			p <- p + geom_text(aes(x=tracking_id,y=0,label=warning,group=1),position=dodge,color='red',stat='identity',vjust=1.5,size=3)
+			p <- p + geom_text(aes(x=tracking_id,y=0,label=warning),position=dodge,color='red',stat='identity',vjust=1.5,size=3)
 		}
 	}
 	#gene_labels = dat$gene_short_name
 	p <- p + scale_x_discrete("",breaks=tracking_ids,labels=gene_labels) + 
-	    opts(axis.text.x=theme_text(hjust=0,angle=-90))
+	    theme(axis.text.x=element_text(hjust=0,angle=-90))
     	
     # p<- p +
     #       geom_bar() +
     #       geom_errorbar(aes(ymin=conf_lo,ymax=conf_hi,group=1),size=0.15) +
     #       facet_wrap('sample_name') +
-    #       opts(axis.text.x=theme_text(hjust=0,angle=-90))
+    #       theme(axis.text.x=element_text(hjust=0,angle=-90))
 	
 	#This does not make immediate sense with the conf_hi and conf_lo values.  Need to figure out appropriate transformation for these
 	#if(logMode)
@@ -450,7 +667,7 @@ setMethod("csVolcano",signature(object="CuffFeatureSet"), .volcano)
         p <- p + ylab("FPKM")
     }
 	
-	#p <- p + opts(legend.position = "none")
+	#p <- p + theme(legend.position = "none")
 	
 	#Default cummeRbund colorscheme
 	p<-p + scale_fill_hue(l=50,h.start=200) + scale_color_hue(l=50,h.start=200)
@@ -464,7 +681,7 @@ setMethod("csVolcano",signature(object="CuffFeatureSet"), .volcano)
 
 setMethod("expressionBarplot",signature(object="CuffFeatureSet"),.barplot)
 
-.expressionPlot<-function(object,logMode=FALSE,pseudocount=1.0, drawSummary=FALSE, sumFun=mean_cl_boot, showErrorbars=TRUE,showStatus=TRUE,...){
+.expressionPlot<-function(object,logMode=FALSE,pseudocount=1.0, drawSummary=FALSE, sumFun=mean_cl_boot, showErrorbars=TRUE,showStatus=TRUE,replicates=FALSE,...){
 	quant_types<-c("OK","FAIL","LOWDATA","HIDATA","TOOSHORT")
 	quant_types<-factor(quant_types,levels=quant_types)
 	quant_colors<-c("black","red","blue","orange","green")
@@ -472,24 +689,39 @@ setMethod("expressionBarplot",signature(object="CuffFeatureSet"),.barplot)
 	
 	dat<-fpkm(object)
 	colnames(dat)[1]<-"tracking_id"
+	
+	if(replicates){
+		repDat<-repFpkm(object)
+		repDat$replicate<-as.factor(repDat$replicate)
+		colnames(repDat)[1]<-"tracking_id"
+	}
+	
 	if(logMode)
 	{
 		dat$fpkm <- dat$fpkm + pseudocount
 		dat$conf_hi <- dat$conf_hi + pseudocount
 		dat$conf_lo <- dat$conf_lo + pseudocount
+		
+		if(replicates){
+			repDat$fpkm<-repDat$fpkm + pseudocount
+		}
 	}
 	p <- ggplot(dat)
 	#dat$fpkm<- log10(dat$fpkm+pseudocount)
-	p <- p + 
-			geom_line(aes(x=sample_name,y=fpkm,group=tracking_id))
+	p <- p + geom_line(aes(x=sample_name,y=fpkm,group=tracking_id,color=tracking_id))
+	
+	if(replicates){
+		p <- p + geom_point(aes(x=sample_name,y=fpkm,color=tracking_id),size=2.5,shape=18,data=repDat)
+	}
+	
 	if (showErrorbars)
 	{
 		p <- p +
-				geom_errorbar(aes(x=sample_name, ymin=conf_lo,ymax=conf_hi,group=tracking_id),width=0.25)
+				geom_errorbar(aes(x=sample_name, ymin=conf_lo,ymax=conf_hi,color=tracking_id,group=tracking_id),width=0.25)
 	}
 	
 	if(showStatus){
-		p <- p+ geom_point(aes(x=sample_name,y=fpkm,shape=quant_status,color=quant_status))
+		p <- p+ geom_point(aes(x=sample_name,y=fpkm,shape=quant_status))
 	}
 	
 	if (logMode)
@@ -510,7 +742,7 @@ setMethod("expressionBarplot",signature(object="CuffFeatureSet"),.barplot)
 	}
 	
 	#Recolor quant flags
-	p<- p+ scale_colour_manual(name='quant_status',values=quant_colors)
+	#p<- p + scale_colour_manual(name='quant_status',values=quant_colors)
 	
 	p
 }
@@ -545,10 +777,13 @@ setMethod("expressionPlot",signature(object="CuffFeatureSet"),.expressionPlot)
 #	c
 #}
 
-.cluster<-function(object, k, pseudocount=1, ...){
-	library(cluster)
+.cluster<-function(object, k, logMode=T, pseudocount=1,...){
+	require(cluster)
 	m<-as.data.frame(fpkmMatrix(object))
 	m<-m[rowSums(m)>0,]
+	if(logMode){
+		m<-log10(m+pseudocount)
+	}
 	n<-JSdist(makeprobs(t(m)))
 	clusters<-pam(n,k, ...)
 	#clsuters<-pamk(n,krange=2:20)
@@ -559,6 +794,19 @@ setMethod("expressionPlot",signature(object="CuffFeatureSet"),.expressionPlot)
 
 setMethod("csCluster",signature(object="CuffFeatureSet"),.cluster)
 
+.ratioCluster<-function(object,k,ratioTo=NULL,...){
+	require(cluster)
+	m<-as.data.frame(fpkmMatrix(object))
+	#TODO: ensure that ratioTo is in colnames(fpkmMatrix(object))
+	m.ratio<-m/m[[ratioTo]]
+	m.log.ratio<-log2(m.ratio)
+	n<-dist(m.log.ratio)
+	clusters<-pam(n,k,...)
+	class(clusters)<-"list"
+	clusters$ratio<-m
+	clusters
+}
+
 csClusterPlot<-function(clustering,pseudocount=1.0,drawSummary=TRUE, sumFun=mean_cl_boot){
 	m<-clustering$fpkm+pseudocount
 	m$ids<-rownames(clustering$fpkm)
@@ -573,8 +821,12 @@ csClusterPlot<-function(clustering,pseudocount=1.0,drawSummary=TRUE, sumFun=mean
 	c
 }
 
-.dendro<-function(object,logMode=T,pseudocount=1){
-	fpkmMat<-fpkmMatrix(object)
+.dendro<-function(object,logMode=T,pseudocount=1,replicates=FALSE){
+	if(replicates){
+		fpkmMat<-repFpkmMatrix(object)
+	}else{
+		fpkmMat<-fpkmMatrix(object)
+	}
 	if(logMode){
 		fpkmMat<-log10(fpkmMat+pseudocount)
 	}
@@ -589,6 +841,18 @@ csClusterPlot<-function(clustering,pseudocount=1.0,drawSummary=TRUE, sumFun=mean
 
 setMethod("csDendro",signature(object="CuffFeatureSet"),.dendro)
 
+#.csNMF<-function(object,rank,method,...){
+#	#Non-negative Matrix Factorization
+#	require(NMF)
+#	#Retrieve FPKM values
+#	fpkms<-fpkmMatrix(object)
+#	
+#	#Remove rows with sum()==0
+#	fpkms<-fpkms[rowSums(fpkms)>0,]
+#	
+#}
+#
+#setMethod("csNMF",signature(object="CuffFeatureSet"),.csNMF)
 
 ##Takes as first argument the object returned from csCluster (a modified 'cluster' list)
 #.clusterPlot<-function(clusters, pseudocount=1, ...){
@@ -605,17 +869,23 @@ setMethod("csDendro",signature(object="CuffFeatureSet"),.dendro)
 #	c
 #}
 
-.density<-function(object, logMode = TRUE, pseudocount=1.0, labels, features=FALSE, ...){
-	dat<-fpkm(object,features=features)
+.density<-function(object, logMode = TRUE, pseudocount=0.0, labels, features=FALSE, replicates=FALSE,...){
+	if(replicates){
+		dat<-repFpkm(object,features=features)
+		colnames(dat)[colnames(dat)=="rep_name"]<-"condition"
+	}else{
+		dat<-fpkm(object,features=features)
+		colnames(dat)[colnames(dat)=="sample_name"]<-"condition"
+	}
 	if(logMode) dat$fpkm<-dat$fpkm+pseudocount
 	p<-ggplot(dat)
 	if(logMode) {
-		p<-p+geom_density(aes(x= log10(fpkm),group=sample_name,color=sample_name,fill=sample_name),alpha=I(1/3))
+		p<-p+geom_density(aes(x= log10(fpkm),group=condition,color=condition,fill=condition),alpha=I(1/3))
 	}else{
-		p<-p+geom_density(aes(x=fpkm,group=sample_name,color=sample_name,fill=sample_name),alpha=I(1/3))
+		p<-p+geom_density(aes(x=fpkm,group=condition,color=condition,fill=condition),alpha=I(1/3))
 	}
 	
-	#p<-p + opts(title=object@tables$mainTable)
+	#p<-p + labs(title=object@tables$mainTable)
 	
 	#Default cummeRbund colorscheme
 	p<-p + scale_fill_hue(l=50,h.start=200) + scale_color_hue(l=50,h.start=200)
@@ -652,4 +922,4 @@ setMethod("csDensity",signature(object="CuffFeatureSet"),.density)
 	1-res
 }
 
-setMethod("csSpecificity",signature(object="CuffFeatureSet"),.specificity)
\ No newline at end of file
+setMethod("csSpecificity",signature(object="CuffFeatureSet"),.specificity)
