@@ -61,6 +61,59 @@ makeprobs<-function(a){
 	b
 }
 
+.plotmatrix <- function (data, hexbin=FALSE, mapping = aes())
+#Modified from original ggplot2 plotmatrix
+{
+	grid <- expand.grid(x = 1:ncol(data), y = 1:ncol(data))
+	grid <- subset(grid, x != y)
+	all <- do.call("rbind", lapply(1:nrow(grid), function(i) {
+						xcol <- grid[i, "x"]
+						ycol <- grid[i, "y"]
+						data.frame(xvar = names(data)[ycol], yvar = names(data)[xcol], 
+								x = data[, xcol], y = data[, ycol], data)
+					}))
+	all$xvar <- factor(all$xvar, levels = names(data))
+	all$yvar <- factor(all$yvar, levels = names(data))
+	densities <- do.call("rbind", lapply(1:ncol(data), function(i) {
+						data.frame(xvar = names(data)[i], yvar = names(data)[i], 
+								x = data[, i])
+					}))
+	mapping <- plyr::defaults(mapping, aes_string(x = "x", y = "y"))
+	class(mapping) <- "uneval"
+	p <-ggplot(all) + facet_grid(xvar ~ yvar)#, scales = "free")
+	
+	if(hexbin){ 
+		p<- p + geom_hex(mapping,size=1.5,na.rm = TRUE) 
+	}else{
+		p<- p + geom_point(mapping,alpha=0.2,size=0.8,na.rm=TRUE)
+	}
+	
+	p<- p + stat_density(aes(x = x, 
+					y = ..scaled.. * diff(range(x)) + min(x)), data = densities, 
+			position = "identity", colour = "grey20", geom = "line")
+	
+	p
+}
+
+#.volcanoMatrix <- function(data){
+#	part1<-data[,c('gene_id','sample_1','sample_2','value_1','value_2','test_stat','p_value','q_value')]
+#	part2<-data.frame(gene_id=part1$gene_id,sample_1=part1$sample_2,sample_2=part1$sample_1,value_1=part1$value_2,value_2=part1$value_1,test_stat=-part1$test_stat,p_value=part1$p_value,q_value=part1$q_value)
+#	data<-rbind(part1,part2)
+#	myLevels<-union(data$sample_1,data$sample_2)
+#	data$sample_1<-factor(data$sample_1,levels=myLevels)
+#	data$sample_2<-factor(data$sample_2,levels=myLevels)
+#	data$log2_fold_change<-log2(data$value_2/data$value_1)
+#	filler<-data.frame(sample_1=factor(myLevels,levels=myLevels),sample_2=factor(myLevels,levels=myLevels),label=as.character(myLevels))
+#	filler$label<-as.character(filler$label)
+#	mapping <- defaults(mapping, aes_string(x = "log2_fold_change", y = "-log10(p_value)"))
+#	class(mapping) <- "uneval"
+#	
+#	p <-ggplot(data) + geom_point(mapping,na.rm=TRUE,size=0.8) + geom_text(aes(x=0,y=15,label=label),data=filler) + facet_grid(sample_1~sample_2)
+#	
+#	p
+#	
+#}
+
 #multiplot <- function(..., plotlist=NULL, cols) {
 #	require(grid)
 #	
